#!/usr/bin/env python3
""" Copyright (c) 2023, Juniper Networks, Inc
    All rights reserved
    This SOFTWARE is licensed under the LICENSE provided in the
    ./LICENCE file. By downloading, installing, copying, or otherwise
    using the SOFTWARE, you agree to be bound by the terms of that
    LICENSE.
"""
_JUNIPER_IFD_ENCAP = {
    0: "Unspecfied",
    1: "Ethernet",
    2: "FDDI",
    3: "Token-Ring",
    4: "PPP",
    5: "Frame-Relay",
    6: "Cisco-HDLC",
    7: "SMDS-DXI",
    8: "ATM-PVC",
    9: "PPP-CCC",
    10: "Frame-Relay-CCC",
    11: "IP-over-IP",
    12: "GRE",
    13: "PIM-Encapsulator",
    14: "PIM-Decapsulator",
    15: "Cisco-HDLC-CCC",
    16: "VLAN-CCC",
    17: "Multilink-PPP",
    18: "Multilink-FR",
    19: "Multilink",
    20: "LSI",
    21: "DFE",
    22: "ATM-CCC-Cell-Relay",
    23: "IPSEC-over-IP",
    24: "GGSN",
    25: "LSI-enabled-PPP-SONET",
    26: "LSI-enabled-CHDLC-SONET",
    27: "PPP-TCC",
    28: "Frame-Relay-TCC",
    29: "Cisco-HDLC-TCC",
    30: "Ethernet-CCC",
    31: "VPN-Loopback-tunnel",
    32: "Extended-VLAN-CCC",
    33: "Ethernet-over-ATM",
    34: "Monitor",
    35: "Ethernet-TCC",
    36: "VLAN-TCC",
    37: "Extended-VLAN-TCC",
    38: "Controller",
    39: "Multilink-FR-UNI-NNI",
    40: "LinkService",
    41: "VPLS",
    42: "VLAN-VPLS",
    43: "Extended-VLAN-VPLS",
    44: "Logical-tunnel",
    45: "General-Services",
    46: "Ethernet-VPLS-over-ATM",
    47: "FR-dedicated-CCC",
    48: "Extended FR-CCC",
    49: "Extended FR-TCC",
    50: "Flexible-FrameRelay",
    51: "GGSN-Inspection",
    52: "Flexible-Ethernet-Services",
    53: "Flow-collection",
    54: "Aggregator",
    55: "LAPD",
    56: "PPPoE",
    57: "PPP-Subordinate",
    58: "Cisco-Hdlc-Subordinate",
    59: "Dynamic-Flow-Capture",
    60: "PIC Peer",
    61: "Single-Link-PPP",
    62: "VOIP",
    63: "Secure-Tunnel",
    64: "Fabric-Member-Ethernet",
    65: "Frame-Relay-Ether-Encap",
    66: "Frame-Relay-Ether-Encap-TCC",
    67: "Extended FR-Ether-Encap-TCC",
    68: "VLAN-VCI-CCC",
    69: "Container",
    70: "Virtual-Chassis-Interface",
    71: "VLAN",
    72: "Management-VLAN",
    73: "SAToP-Encapsulation",
    74: "CESoPSN-Encapsulation",
    75: "Uplink Tunnel",
    76: "Ethernet VPLS over PPP",
    77: "Xconnect Tunnel",
    78: "Inverse Multiplexing for ATM",
    79: "MPU",
    80: "FXS",
    81: "FXO",
    82: "FPC-LOCAL",
    83: "PFE-LOCAL",
    84: "DOCSIS",
    85: "JUNIPER-SERVICES-VLAN",
    86: "UDP Tunnel",
    87: "Fibrechannel",
    88: "Virtual-Server",
    89: "Switch Fabric-Member-Ethernet",
    90: "Pseudowire Service",
    91: "VxLAN Tunnel End Point",
    92: "Satellite Device",
    93: "Remote beb",
    94: "EVPN VxLAN AA",
    95: "SRX-SPU",
    96: "OT",
    97: "Flexible Tunnel",
}

_JUNIPER_IFL_ENCAP = {
    0: "Unspecified",
    1: "Null",
    2: "ATM SNAP",
    3: "ATM NLPID",
    4: "ATM VCMUX",
    5: "ATM LLC",
    6: "ATM PPP VCMUX",
    7: "ATM PPP LLC",
    8: "ATM PPP Funi",
    9: "CCC/ATM",
    10: "FR NLPID",
    11: "FR SNAP",
    12: "FR PPP",
    13: "FR CCC",
    14: "Ethernet",
    15: "802.3 SNAP",
    16: "802.3 LLC",
    17: "PPP",
    18: "Cisco HDLC",
    19: "CCC/PPP",
    20: "IP-in-IP",
    21: "PIM-E",
    22: "GRE",
    23: "GRE PPP",
    24: "PIM-D",
    25: "CCC/HDLC",
    26: "ATM C-NLPID",
    27: "CCC/VLAN",
    28: "ML-PPP",
    29: "ML-FR",
    32: "ATM Cell-relay",
    33: "IPSEC",
    34: "GGSN",
    35: "TCC/ATM",
    36: "TCC/FR",
    37: "TCC/PPP",
    38: "TCC/HDLC",
    39: "CCC/Ethernet",
    40: "VPN-loop tun",
    41: "Ethernet/ATM",
    42: "Extended CCC/VLAN",
    43: "TCC/ATM SNAP",
    44: "Monitor",
    45: "TCC/Ethernet",
    46: "TCC/VLAN",
    47: "Extended TCC/VLAN",
    48: "ML-FR-UNI-NNI",
    49: "VPLS/Ethernet",
    50: "VLAN/VPLS",
    51: "Extended VLAN/VPLS",
    52: "Services",
    53: "VPLS/ATM",
    54: "CCC/FR-PORT",
    55: "ATM MLPPP LLC",
    56: "CCC/Ethernet over ATM",
    57: "VLAN/LT",
    58: "Flow collection",
    59: "Flow aggregation",
    60: "ISDN-LAPD",
    61: "PPPoEoA LLC",
    62: "PPP over Ethernet",
    63: "PPPoE",
    66: "DFC",
    67: "PIC Peer",
    68: "VOIP",
    69: "Secure Tunnel",
    70: "FR-ETHER",
    71: "TCC/FR-ETHER",
    72: "CCC/VLAN Q-in-Q and VCI interworking",
    73: "CCC/TDM-SATOP",
    74: "CCC/TDM-CESoPSN",
    75: "VPLS over FR",
    76: "VPLS over PPP",
    78: "AMT UDP Tunnel",
    79: "Fibrechannel",
    80: "GTP",
    81: "Virtual Server",
    82: "Tether Enet Svcs",
    84: "CAPWAP Tunnel",
    85: "GRE L2",
    86: "VXLAN V4 over FT",
    87: "VXLAN V6 over FT",
    88: "UDP V4 over FT",
    89: "IPV4 over FT",
    255: "Link Specific",
}



#!/usr/bin/env python3
""" Copyright (c) 2023, Juniper Networks, Inc
    All rights reserved
    This SOFTWARE is licensed under the LICENSE provided in the
    ./LICENCE file. By downloading, installing, copying, or otherwise
    using the SOFTWARE, you agree to be bound by the terms of that
    LICENSE.
"""
# stdlib imports
import re
from argparse import ArgumentParser

# local imports
from .juniper_ethernet import JuniperEthernet

# 3rd party imports
from scapy.utils import rdpcap, wrpcap
from scapy.plist import PacketList
from scapy.layers.l2 import Ether, Dot1Q
from scapy.compat import raw


def parse_args():
    """parses arguments
    :return args:
    :type Namespace:
    """
    parser = ArgumentParser()
    parser.add_argument("inpcap", help="path to input pcap file")
    parser.add_argument("outpcap", help="path to output pcap file")
    parser.add_argument(
        "--overwrite",
        action="store_true",
        help="allow output file to be overwritten",
    )
    parser.add_argument(
        "--dmac",
        nargs=1,
        type=str,
        default=["00:00:00:00:00:00"],
        help="dst mac address to add to Ethernet header",
    )
    parser.add_argument(
        "--smac",
        nargs=1,
        type=str,
        default=["00:00:00:00:00:00"],
        help="src mac address to add to Ethernet header",
    )
    parser.add_argument(
        "--vlans",
        nargs=1,
        help="vlan id(s) to add to Ethernet header, example: 1,4",
    )
    args = parser.parse_args()
    return args


def process_optargs(args):
    """checks optional args validity
    :returns smac:
    :type string:
    :returns dmac:
    :type string:
    :returns vlans:
    :type list:
    :raises ValueError: if either vlans, smac, dmac are incorrect
    :raises FileExistsError: if output file already exists
    """
    ofile_exists = False
    try:
        with open(args.outpcap, "r") as outfile:
            ofile_exists = True
    except FileNotFoundError:
        pass

    if ofile_exists:
        with open(args.outpcap, "a") as outfile:
            if not args.overwrite:
                raise FileExistsError("to overwrite file, specify --overwrite argument")

    smac = args.smac[0].lower()
    dmac = args.dmac[0].lower()
    mac_address = re.compile(r"([a-f0-9]{2}:){5}[a-f0-9]{2}")
    if not re.match(mac_address, smac):
        raise ValueError("src mac not in the correct format")
    if not re.match(mac_address, dmac):
        raise ValueError("dst mac not in the correct format")

    vlans = []
    if args.vlans:
        vlans = args.vlans[0].split(",")
    if len(vlans) > 2:
        raise ValueError("max of 2 vlans can be added")

    return (smac, dmac, vlans)


def main():
    """Processes arguments. Reads packets from input pcap file.
    Process each packet popping off the juniper ethernet header.
    If L2 header is missing from inbound packet, a new L2 header is added
    along with any dot1q headers specified.
    Outbound packets always have an L2 header.
    Packets are written to the output pcap file.
    :returns None:
    """
    args = parse_args()
    smac, dmac, vlans = process_optargs(args)
    popped_pkts = 0
    in_pkts = rdpcap(args.inpcap)
    out_pkts = PacketList()
    for in_pkt in in_pkts:
        if not raw(in_pkt)[:3] == b"MGC":
            out_pkts.append(in_pkt)
            continue
        je_pkt = JuniperEthernet(in_pkt.load)
        if je_pkt.l3_payload and vlans:
            # inbound packet with L3 payload, add Ethernet and Dot1Q headers
            out_pkt = Ether(src=smac, dst=dmac)
            for vlan in vlans:
                out_pkt = out_pkt / Dot1Q(vlan=int(vlan))
            out_pkt = out_pkt / je_pkt.payload
        elif je_pkt.l3_payload:
            # inbound packet with L3 payload, add Ethernet header
            out_pkt = Ether(src=smac, dst=dmac) / je_pkt.payload
        else:
            # packet with L2 payload
            out_pkt = je_pkt.payload
        out_pkts.append(out_pkt)
        popped_pkts += 1
    if not popped_pkts:
        raise SystemExit(f"no packets with a Juniper Ethernet header found")
    wrpcap(args.outpcap, out_pkts)



#!/usr/bin/env python3
""" Copyright (c) 2023, Juniper Networks, Inc
    All rights reserved
    This SOFTWARE is licensed under the LICENSE provided in the
    ./LICENCE file. By downloading, installing, copying, or otherwise
    using the SOFTWARE, you agree to be bound by the terms of that
    LICENSE.
"""
# stdlib imports
import logging

# local imports
from .juniper_encap import _JUNIPER_IFD_ENCAP, _JUNIPER_IFL_ENCAP

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
# 3rd party imports
from scapy.fields import (
    ByteField,
    ByteEnumField,
    FieldLenField,
    StrLenField,
    X3BytesField,
    FlagsField,
    PacketListField,
    ConditionalField,
    ThreeBytesField,
)
from scapy.packet import Packet
from scapy.layers.l2 import Ether
from scapy.layers.inet import IP
from scapy.layers.inet6 import IPv6

_JUNIPER_ETHERNET_HEADER_TLVS = {
    1: "JNX_EXT_TLV_IFD_IDX",
    2: "JNX_EXT_TLV_IFD_NAME",
    3: "JNX_EXT_TLV_IFD_MEDIATYPE",
    4: "JNX_EXT_TLV_IFL_IDX",
    5: "JNX_EXT_TLV_IFL_UNIT",
    6: "JNX_EXT_TLV_IFL_ENCAPS",
    7: "JNX_EXT_TLV_TTP_IFD_MEDIATYPE",
    8: "JNX_EXT_TLV_TTP_IFL_ENCAPS",
    9: "JNX_EXT_TLV_IRB_IFL_IDX",
    10: "JNX_EXT_TLV_IRB_L2_INPUT_IFL_IDX",
    11: "JNX_EXT_TLV_IRB_L2_OUTPUT_IFL_IDX",
    12: "JNX_EXT_TLV_JPTAP_META",
}

_L3_PAYLOAD = {
    2: "IP",
    6: "IPv6",
}

class JuniperEthernetExtensionTLV(Packet):
    name = "Juniper Ethernet - Generic Extension TLV"
    fields_desc = [
        ByteEnumField("type", None, _JUNIPER_ETHERNET_HEADER_TLVS),
        FieldLenField("len", None, length_of="value", fmt="b"),
        StrLenField("value", "", length_from=lambda pkt: pkt.len),
    ]

    def extract_padding(self, p):
        return b"", p

    registered_je_tlv = {}

    @classmethod
    def register_variant(cls):
        cls.registered_je_tlv[cls.type.default] = cls

    @classmethod
    def dispatch_hook(cls, pkt=None, *args, **kargs):
        if pkt:
            tmp_type = ord(pkt[:1])
            return cls.registered_je_tlv.get(tmp_type, cls)
        return cls


class JuniperEthernetInterfaceIndexTLV(JuniperEthernetExtensionTLV):
    name = "Juniper Ethernet Interface Index Extension TLV"
    fields_desc = [
        ByteEnumField("type", 1, _JUNIPER_ETHERNET_HEADER_TLVS),
        FieldLenField("len", None, length_of="value", fmt="b"),
        StrLenField("value", None, length_from=lambda x: x.len),
    ]

    def post_dissection(self, pkt):
        if pkt.fields["value"]:
            pkt.fields["value"] = int.from_bytes(
                pkt.fields["value"], byteorder="little"
            )


class JuniperEthernetInterfaceMediaTypeTLV(JuniperEthernetExtensionTLV):
    name = "Juniper Ethernet Interface Media Type Extension TLV"
    fields_desc = [
        ByteEnumField("type", 3, _JUNIPER_ETHERNET_HEADER_TLVS),
        FieldLenField("len", None, length_of="value", fmt="b"),
        StrLenField("value", None, length_from=lambda x: x.len),
    ]

    def post_dissection(self, pkt):
        if pkt.fields["value"]:
            encap_num = int.from_bytes(pkt.fields["value"], byteorder="little")
            pkt.fields["value"] = _JUNIPER_IFD_ENCAP[encap_num]


class JuniperEthernetLogicalInterfaceIndexTLV(JuniperEthernetExtensionTLV):
    name = "Juniper Ethernet Logical Interface Index Extension TLV"
    fields_desc = [
        ByteEnumField("type", 4, _JUNIPER_ETHERNET_HEADER_TLVS),
        FieldLenField("len", None, length_of="value", fmt="b"),
        StrLenField("value", None, length_from=lambda x: x.len),
    ]

    def post_dissection(self, pkt):
        if pkt.fields["value"]:
            pkt.fields["value"] = int.from_bytes(
                pkt.fields["value"], byteorder="little"
            )


class JuniperEthernetLogicalInterfaceUnitTLV(JuniperEthernetExtensionTLV):
    name = "Juniper Ethernet Logical Interface Unit Extension TLV"
    fields_desc = [
        ByteEnumField("type", 5, _JUNIPER_ETHERNET_HEADER_TLVS),
        FieldLenField("len", None, length_of="value", fmt="b"),
        StrLenField("value", None, length_from=lambda x: x.len),
    ]

    def post_dissection(self, pkt):
        if pkt.fields["value"]:
            pkt.fields["value"] = int.from_bytes(
                pkt.fields["value"], byteorder="little"
            )


class JuniperEthernetLogicalInterfaceEncapsulationTLV(JuniperEthernetExtensionTLV):
    name = "Juniper Ethernet Logical Interface Encapsulation Extension TLV"
    fields_desc = [
        ByteEnumField("type", 6, _JUNIPER_ETHERNET_HEADER_TLVS),
        FieldLenField("len", None, length_of="value", fmt="b"),
        StrLenField("value", None, length_from=lambda x: x.len),
    ]

    def post_dissection(self, pkt):
        if pkt.fields["value"]:
            encap_num = int.from_bytes(pkt.fields["value"], byteorder="little")
            pkt.fields["value"] = _JUNIPER_IFL_ENCAP[encap_num]


class JuniperEthernetTtpInterfaceMediaType(JuniperEthernetExtensionTLV):
    name = "Juniper Ethernet TTP Interface Media Type Extension TLV"
    fields_desc = [
        ByteEnumField("type", 7, _JUNIPER_ETHERNET_HEADER_TLVS),
        FieldLenField("len", None, length_of="value", fmt="b"),
        StrLenField("value", None, length_from=lambda x: x.len),
    ]

    def post_dissection(self, pkt):
        if pkt.fields["value"]:
            encap_num = int.from_bytes(pkt.fields["value"], byteorder="little")
            pkt.fields["value"] = _JUNIPER_IFD_ENCAP[encap_num]


class JuniperEthernetTtpLogicalInterfaceEncapsulation(JuniperEthernetExtensionTLV):
    name = "Juniper Ethernet TTP Logical Interface Encapsulation Extension TLV"
    fields_desc = [
        ByteEnumField("type", 8, _JUNIPER_ETHERNET_HEADER_TLVS),
        FieldLenField("len", None, length_of="value", fmt="b"),
        StrLenField("value", None, length_from=lambda x: x.len),
    ]

    def post_dissection(self, pkt):
        if pkt.fields["value"]:
            encap_num = int.from_bytes(pkt.fields["value"], byteorder="little")
            pkt.fields["value"] = _JUNIPER_IFL_ENCAP[encap_num]


class JuniperEthernetIrbLogicalInterfaceIndexTLV(JuniperEthernetExtensionTLV):
    name = "Juniper Ethernet IRB Logical Interface Index Extension TLV"
    fields_desc = [
        ByteEnumField("type", 9, _JUNIPER_ETHERNET_HEADER_TLVS),
        FieldLenField("len", None, length_of="value", fmt="b"),
        StrLenField("value", None, length_from=lambda x: x.len),
    ]

    def post_dissection(self, pkt):
        if pkt.fields["value"]:
            pkt.fields["value"] = int.from_bytes(
                pkt.fields["value"], byteorder="little"
            )


class JuniperEthernetIrbL2InputLogicalInterfaceIndexTLV(JuniperEthernetExtensionTLV):
    name = "Juniper Ethernet IRB Layer2 Input Logical Interface Index Extension TLV"
    fields_desc = [
        ByteEnumField("type", 10, _JUNIPER_ETHERNET_HEADER_TLVS),
        FieldLenField("len", None, length_of="value", fmt="b"),
        StrLenField("value", None, length_from=lambda x: x.len),
    ]

    def post_dissection(self, pkt):
        if pkt.fields["value"]:
            pkt.fields["value"] = int.from_bytes(
                pkt.fields["value"], byteorder="little"
            )


class JuniperEthernetIrbL2OutputLogicalInterfaceIndexTLV(JuniperEthernetExtensionTLV):
    name = "Juniper Ethernet IRB Layer2 Output Logical Interface Index Extension TLV"
    fields_desc = [
        ByteEnumField("type", 11, _JUNIPER_ETHERNET_HEADER_TLVS),
        FieldLenField("len", None, length_of="value", fmt="b"),
        StrLenField("value", None, length_from=lambda x: x.len),
    ]

    def post_dissection(self, pkt):
        if pkt.fields["value"]:
            pkt.fields["value"] = int.from_bytes(
                pkt.fields["value"], byteorder="little"
            )


# class JuniperEthernetJptapMetaTLV(JuniperEthernetExtensionTLV):
#    name = "Juniper Ethernet Jptap Meta Extension TLV"
#    fields_desc = [
#        ByteEnumField("type", 12, _JUNIPER_ETHERNET_HEADER_TLVS),
#        ByteField("len", ),
#        NBytesField("value", 0, ),
#    ]


class JuniperEthernet(Packet):
    name = "JuniperEthernet"
    fields_desc = [
        X3BytesField("magic_number", 0x4D4743),
        FlagsField("extensions_present", 0, 6, ["f0", "f1", "f2", "f3", "f4", "f5"]),
        FlagsField("l2_header_presence", 0, 1, ["f0"]),
        FlagsField("direction", 0, 1, ["f0"]),
        FieldLenField("extensions_length", None, length_of="tlv_objects"),
        PacketListField(
            "tlv_objects",
            [],
            JuniperEthernetExtensionTLV,
            length_from=lambda pkt: pkt.extensions_length,
        ),
        ConditionalField(
            ByteEnumField("l3_payload", None, _L3_PAYLOAD),
            lambda pkt: pkt.l2_header_presence == "f0",
        ),
        ConditionalField(
            ThreeBytesField("padding", None),
            lambda pkt: pkt.l2_header_presence == "f0",
        ),
    ]

    def guess_payload_class(self, payload):
        if not self.l2_header_presence:
            return Ether
        elif self.l3_payload == 2:
            return IP
        elif self.l3_payload == 6:
            return IPv6
